//ADAPTED
/*
    This file is part of Rekall.
    Copyright (C) 2013-2015

    Project Manager: Clarisse Bardiot
    Development & interactive design: Guillaume Jacquemin & Guillaume Marais (http://www.buzzinglight.com)

    This file was written by Guillaume Jacquemin.

    Rekall is a free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

function Rekall(options) {
    this.sortings = new Object();
    this.sortings["horizontal"] = new Sorting("Time");
    this.sortings["colors"] = new Sorting("Rekall->Type", false, "");
    this.timeline = new Timeline();
    this.videoPlayer = undefined;

    function getDefaultBaseUrl() {
        var oldURL = window.document.location + "";
        var url = oldURL;
        var index = 0;
        url = url.replace("pro-index.html", "");
        url = url.replace("online-index.html", "");
        url = url.replace("index.html", "");
        index = oldURL.indexOf('?');
        if (index == -1)
            index = oldURL.indexOf('#');
        if (index != -1)
            url = oldURL.substring(0, index);
        if (!url.endsWith("/"))
            url = url + "/";

        return url;
    }

    let defaultSettings = {
        _allowProjectDeletion: false,
        baseUrl: getDefaultBaseUrl()
    }

    this.Settings = $.extend({}, defaultSettings, options);
}

Rekall.prototype.allowProjectDeletion = function () {
    if (arguments.length <= 0 && 'boolean' !== typeof arguments[0]) {
        return this.Settings._allowProjectDeletion;
    }

    if (arguments[0]) {
        this.Settings._allowProjectDeletion = true;
        $('#popupSettingsBtnDelete').attr('display', null);
    } else {
        this.Settings._allowProjectDeletion = false;
        $('#popupSettingsBtnDelete').hide();
    }
    this.Settings._allowProjectDeletion = arguments[0];
}

Rekall.prototype.publish = function (eventName, data){
    return window.app.rekall.Rekall('pubSub').publish(eventName, data);
}

Rekall.prototype.loadXMLFile = function () {
    var oldURL = window.document.location + "";
    var index = 0;
    var url = oldURL;
    url = url.replace("pro-index.html", "");
    url = url.replace("online-index.html", "");
    url = url.replace("index.html", "");
    index = oldURL.indexOf('?');
    if (index == -1)
        index = oldURL.indexOf('#');
    if (index != -1)
        url = oldURL.substring(0, index);
    if (!url.endsWith("/"))
        url = url + "/";

    if (!this.Settings._allowProjectDeletion) {
        $('#popupSettingsBtnDelete').hide();
    }

    this.baseUrl = url;

    let that = this;
    this.publish('roulette.started');
    $.ajax("php/file.php?r=project.xml", {
        type: "GET",
        cache: false,
        data: {"rand": Math.random()},
        dataType: "xml",
        success: function (xml) {
            if ((xml == null) || (xml == undefined)) {
                that.publish('alert', {
                    message: "Your project is unreachable. Did you move the folder to an other location?",
                    buttons: 60
                });
            } else {
                $(xml).find('project').each(function () {
                    //if($(this).find('document').length == 0) {
                    //	openAlert("Start by adding files to your project.", 60);
                    //} else {
                    if (that.project == undefined)
                        that.project = new Project(url, that);
                    that.project.loadXML($(this));
                    //}
                });
            }
            that.publish('roulette.terminated', {});
        },
        error: function () {
            $.ajax("file/project.xml", {
                type: "GET",
                cache: false,
                data: {"rand": Math.random()},
                dataType: "xml",
                success: function (xml) {
                    if ((xml == null) || (xml == undefined)) {
                        that.publish('alert', {
                            message: "Your project is unreachable. Did you move the folder to an other location?",
                            buttons: 60
                        });
                    } else {
                        $(xml).find('project').each(function () {
                            if ($(this).find('document').length == 0) {
                                that.publish('alert', {
                                    message: "Start by adding files to your project.",
                                    buttons: 60
                                });
                            } else {
                                if (that.project == undefined)
                                    that.project = new Project(url, that);
                                that.project.loadXML($(this));
                            }
                        });
                    }
                    that.publish('roulette.terminated', {});
                },
                error: function () {
                    that.publish('roulette.terminated', {});
                }
            });
            that.publish('roulette.terminated');
        }
    });
}

Rekall.prototype.ensureVideoPlayerCreated = function (url, tech) {
    if (undefined !== this.videoPlayer)
        return false;

    let that = this;
    let techOrder = ["vimeo", "youtube", "html5"];
    if (url.indexOf("youtube") >= 0) tech = "youtube";
    if (url.indexOf("youtu.be") >= 0) tech = "youtube";
    if (url.indexOf("vimeo") >= 0) tech = "vimeo";
    if (url.indexOf("dailymotion") >= 0) tech = "dailymotion";
    if (url.indexOf("dai.ly") >= 0) tech = "dailymotion";
    if ((tech !== "") && (tech !== undefined))
        techOrder = [tech, "html5"];

    //Video
    this.videoPlayer = videojs("video", {
        "techOrder": techOrder,
        "controls": true,
        "autoplay": false,
        "loop": "false",
        "preload": "auto",
        "sources": [
            {
                "type": "video/" + tech,
                "src": url
            }
        ]
    }, function () {
        $(".vjs-fullscreen-control").hide();

        that.videoPlayer.on("durationchange", function () {
            console.log("durationchange");
            that.project.analyse();
            that.videoPlayer.markers.initialize();
        });
        that.videoPlayer.on("ended", function () {
            console.log("ended");
        });
        that.videoPlayer.on("error", function () {
            console.log("error");
        });
        that.videoPlayer.on("firstplay", function () {
            console.log("firstplay");
        });
        that.videoPlayer.on("fullscreenchange", function () {
            console.log("fullscreenchange");
        });
        that.videoPlayer.on("loadedalldata", function () {
            console.log("loadedalldata");
        });
        that.videoPlayer.on("loadeddata", function () {
            console.log("loadeddata");
        });
        that.videoPlayer.on("loadedmetadata", function () {
            console.log("loadedmetadata");
            that.project.analyse();
        });
        that.videoPlayer.on("loadstart", function () {
            console.log("loadstart");
        });
        that.videoPlayer.on("pause", function () {
            console.log("pause");
        });
        that.videoPlayer.on("play", function () {
            console.log("play");
        });
        that.videoPlayer.on("progress", function () {
            console.log("progress");
        });
        that.videoPlayer.on("seeked", function () {
            console.log("seeked");
        });
        that.videoPlayer.on("seeking", function () {
            console.log("seeking");
        });
        that.videoPlayer.on("timeupdate", function () {
            console.log("timeupdate");
            that.timeline.update(that.videoPlayer.currentTime());
        });
        that.videoPlayer.on("volumechange", function () {
            console.log("volumechange");
        });
        that.videoPlayer.on("waiting", function () {
            console.log("waiting");
        });
        that.videoPlayer.on("resize", function () {
            console.log("resize");
        });
    });

    return true;
}
