<?php

namespace App\Form;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Validator\Constraints\UserPassword;
use Symfony\Component\Validator\Constraints\Length;
use Symfony\Component\Validator\Constraints\NotBlank;

class EditPasswordFormType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('current_password', PasswordType::class, [
                    'mapped' => false,
                    'required' => true,
                    'constraints' => [
                        new NotBlank(['message' => 'password.not_blank']),
                        new UserPassword(['message' => "password.invalid"])
                    ],
                    'label' => 'user.password.current'
                ])
            ->add('plainPassword', RepeatedType::class, [
                'type' => PasswordType::class,
                'first_options' => [
                    'attr' => ['autocomplete' => 'new-password'],
                    'constraints' => [
                        new NotBlank([
                            'message' => 'password.not_blank',
                        ]),
                        new Length([
                            'min' => 6,
                            'minMessage' => 'password.min_characters',
                            'max' => 4096,
                        ]),
                    ],
                    'label' => 'reset_password.new_password',
                ],
                'second_options' => [
                    'attr' => ['autocomplete' => 'new-password'],
                    'label' => 'reset_password.repeat_password',
                ],
                'invalid_message' => 'password.match',
                'mapped' => false,
            ])
            ->add(
                'change_password',
                SubmitType::class,
                ['label' => 'user.password.change_password']
            )
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([]);
    }
}
