<?php

namespace App\Form;

use App\Entity\User;
use Gregwar\CaptchaBundle\Type\CaptchaType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\CallbackTransformer;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\IsTrue;
use Symfony\Component\Validator\Constraints\Length;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints\Type;

class RegistrationFormType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add(
                'firstName',
                TextType::class,
                [
                    'constraints' => [new NotBlank(['message' => 'firstName.not_blank'])],
                    'label' => 'registration.firstName'
                ]
            )
            ->add(
                'lastName',
                TextType::class,
                [
                    'constraints' => [new NotBlank(['message' => 'lastName.not_blank'])],
                    'label' => 'registration.lastName'
                ]
            )
            ->add(
                'email',
                EmailType::class,
                [
                    'constraints' => [new NotBlank(['message' => 'email.not_blank'])],
                    'label' => 'general.email'
                ]
            )
            ->add('plainPassword', RepeatedType::class, [
                'type' => PasswordType::class,
                'required' => true,
                'constraints' => [
                    new NotBlank(['message' => 'password.not_blank']),
                    new Length([
                        'min' => 6,
                        'minMessage' => 'password.min_characters',
                        // max length allowed by Symfony for security reasons
                        'max' => 4096,
                    ])
                ],
                'first_options' => ['label' => 'registration.password.first'],
                'second_options' => ['label' => 'registration.password.second'],
                'invalid_message' => 'password.match'
            ])
            ->add(
                'captcha',
                CaptchaType::class,
                [
                    'label' => 'registration.captcha',
                    'required' => true,
                    'reload' => true,
                    'as_url' => true,
                ]
            )
            ->add(
                'acceptGeneralConditions',
                CheckboxType::class,
                [
                    'constraints' => [
                        new IsTrue(['message' => 'agreeTerms']),
                    ],
                    'row_attr' => ['class' => 'form-group d-flex flex-row m-auto mb-4 col-6'],
                    'label' => 'registration.agreeTerms'
                ]
            )
            ->add(
                'submit',
                SubmitType::class,
                ['label' => 'registration.register']
            )
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => User::class,
        ]);
    }
}
