<?php

namespace App\Retriever;

use App\Entity\Capsule;

class ProjectRetriever
{
    private Capsule $capsule;

    public function __construct(Capsule $capsule)
    {
        $this->capsule = $capsule;
    }

    private static function getDomDocument(): \DOMDocument
    {
        chdir('../legacy/');

        $dom_xml = new \DOMDocument();
        $dom_xml->preserveWhiteSpace = false;
        return $dom_xml;
    }

    public function getVideoUrl(): string
    {
        $dom_xml = self::getDomDocument();

        $dom_xml->load($this->capsule->getLinkPath() . '/file/project.xml');
        $video_node = $dom_xml->getElementsByTagName('video')->item(0);

        if ($video_node === null) {
            throw new \Exception("Video node could not be found in XML project file");
        }

        return $video_node->getAttribute('url');
    }

    public function getVideoId(string $video_url): string
    {
        if (strpos($video_url, "vimeo")) {
            return $this->getVimeoVideoIdFromUrl($video_url);
        }
        return $this->getYoutubeVideoIdFromUrl($video_url);
    }

    private function getYoutubeVideoIdFromUrl(string $video_url): string
    {
        $pattern = "#(?<=v=|v\/|vi=|vi\/|youtu.be\/|embed/)[a-zA-Z0-9_-]{11}#";
        $result = preg_match($pattern, $video_url, $matches);

        $video_id = "";

        if (false !== $result && 0 !== $result) {
            $video_id = $matches[0];
        }

        return $video_id;
    }

    private function getVimeoVideoIdFromUrl(string $video_url): string
    {
        $video_id = "";
        $pattern = "/https?:\/\/(?:www\.)?vimeo.com\/(?:channels\/(?:\w+\/)?|groups\/([^\/]*)\/videos\/|album\/(\d+)\/video\/|)(\d+)(?:$|\/|\?)/"; //phpcs:ignore

        if (preg_match($pattern, $video_url, $id)) {
            $video_id = $id[3];
        }
        return $video_id;
    }
}
